library(worldfootballR)
library(dplyr)
library(tidyr)
library(gt)
library(poisbinom)

filename_tab <- "BL2324_xPTSTabelle.rds"
#' prfe bei einer abgeschlossenen Saison die Variable Round,
#' je nach Liga knnte es Play-Off Spiele geben. 
league_matches <- fb_match_results(country = "GER", gender = "M", season_end_year = 2024,
                                   tier = "1st") %>% arrange(Date, Time) %>%
  mutate(game_id = row_number(), Wk = as.numeric(Wk)) # %>% filter(Round == "Regular season")




# neue Funktion definieren, die die Siegwahrscheinlichkeiten berechnet
winprob <- function(team1, team2) {
 lt1 <- length(team1)
 lt2 <- length(team2)
 resultp <- c(0, 0, 0)
 names(resultp) <- c("Team 1", "X", "Team 2")
 if (lt1 == 0 & lt2 == 0) {
  resultp[[2]] <- 1
  return(resultp)
 }
 
 if (lt1 > lt2) {
  gt1 <- dpoisbinom(seq(from = 0, to = lt2), team1)
  gt1[[lt2 + 2]] <- 1 - sum(gt1)
  gt2 <- dpoisbinom(seq(from = 0, to = lt2), team2)
  
  resultp[[2]] <- sum(gt1[-(lt2 + 2)] * gt2)
  if (lt2 > 0) {
   resultp[[3]] <- sum(vapply(2:(lt2 + 1), function(x) gt2[[x]] * sum(gt1[seq_len(x - 1)]),
                              FUN.VALUE = 2))
  }
  resultp[[1]] <- 1 - sum(resultp)
  
 } else {
  gt1 <- dpoisbinom(seq(from = 0, to = lt1), team1)
  gt2 <- dpoisbinom(seq(from = 0, to = lt1), team2)
  gt2[[lt1 + 2]] <- 1 - sum(gt2)
  
  resultp[[2]] <- sum(gt1 * gt2[-(lt1 + 2)])
  if (lt1 > 0) {
   resultp[[1]] <- sum(vapply(2:(lt1 + 1), function(x) gt1[[x]] * sum(gt2[seq_len(x - 1)]),
                              FUN.VALUE = 2))
  }
  resultp[[3]] <- 1 - sum(resultp)
 }
 resultp
}

#### erstmaliges Aufsetzen ####
# die Schussdaten werden runtergeladen 
shot_multiple_matches <- fb_match_shooting((league_matches %>% filter(!is.na(Home_xG)))$MatchURL) %>%
 mutate(xG = as.numeric(xG), Date = as.Date(Date))

# Spiel ID wird zum Schuss Datensatz hinzugefgt
shot_multiple_matches <- league_matches %>% pivot_longer(Home | Away, values_to = "Squad", names_to = "location") %>%
 select(Date, Squad, game_id) %>%
 right_join(shot_multiple_matches, by = c("Date", "Squad"), multiple = "all")

# Finden von Doppel- und Mehrfachchancen
ms_situation <- which(shot_multiple_matches$Event_SCA_1 == "Shot" |
                       (shot_multiple_matches$Event_SCA_2 == "Shot" &
                         !shot_multiple_matches$Event_SCA_1 %in% c("Fouled", "Pass (Dead)")))
# xG Wert fr Mehrfachchancen bestimmen
xg_temp <- shot_multiple_matches$xG
for (i in ms_situation) {
 xg_temp[[i]] <- 1 - (1 - xg_temp[[i-1]]) * (1 - xg_temp[[i]])
}
# Datensatz mit den neuen xG Werten fr Mehrfachchancen updaten
shot_multiple_matches <- shot_multiple_matches %>% mutate(xG = xg_temp) %>%
 filter(!row_number() %in% (ms_situation - 1))

# xPts zu dem Datensatz mit den Ligaspielen hinzufgen
league_matches <- league_matches %>%
 mutate(vapply(game_id, function(id) {
  if (is.na((league_matches %>% filter(game_id == id))$Home_xG)) {
   return(c("Home_xPTS" = NA_integer_, "Away_xPTS" = NA_integer_))
  }
  wp <- winprob((shot_multiple_matches %>% filter(game_id == id & Home_Away == "Home"))$xG,
                (shot_multiple_matches %>% filter(game_id == id & Home_Away == "Away"))$xG)
  c("Home_xPTS" = 3*wp[[1]] + wp[[2]], "Away_xPTS" = 3*wp[[3]] + wp[[2]])
 }, FUN.VALUE = numeric(2)) %>% t() %>% as_tibble())

# der Datensatz mit den Tabellen wird erstellt
league_table <- league_matches %>%
 mutate(Pts_Home = ifelse(HomeGoals > AwayGoals, 3,
                          ifelse(AwayGoals > HomeGoals, 0, 1)),
        Pts_Away = ifelse(Pts_Home == 3, 0, ifelse(Pts_Home == 1, 1, 3))) %>%
 select(Date, Time, game_id, Wk, Home, HomeGoals, Home_xG, Home_xPTS, Pts_Home,
        Away, AwayGoals, Away_xG, Away_xPTS, Pts_Away)
league_table <- league_table %>%
 pivot_longer(cols = c("Home", "Away"), values_to = "Squad", names_to = "Location") %>%
 mutate(GF = ifelse(Location == "Home", HomeGoals, AwayGoals),
        GA = ifelse(Location == "Home", AwayGoals, HomeGoals),
        xGF = ifelse(Location == "Home", Home_xG, Away_xG),
        xGA = ifelse(Location == "Home", Away_xG, Home_xG),
        Pts = ifelse(Location == "Home", Pts_Home, Pts_Away),
        xPts = ifelse(Location == "Home", Home_xPTS, Away_xPTS)) %>%
 select(Wk, Squad, Location, GF, GA, Pts, xGF, xGA, xPts)
league_table <- league_table %>% group_by(Squad) %>%
 mutate(GF = cumsum(GF), GA = cumsum(GA), GDiff = GF - GA,
        Pts = cumsum(Pts), xGF = cumsum(xGF), xGA = cumsum(xGA),
        xGDiff = xGF - xGA, xPts = cumsum(xPts)) %>% ungroup() %>%
 arrange(Wk, desc(Pts), desc(GDiff), desc(GF), desc(xPts))

# Abspeichern der Tabellendaten
saveRDS(league_table, filename_tab)