library(worldfootballR)
library(dplyr)
library(tidyr)
library(gt)
library(poisbinom)

filename_tab <- "BL2324_xPTSTabelle.rds"
#' prfe bei einer abgeschlossenen Saison die Variable Round,
#' je nach Liga knnte es Play-Off Spiele geben. 
league_matches <- fb_match_results(country = "GER", gender = "M", season_end_year = 2024,
                                   tier = "1st") %>% arrange(Date, Time) %>%
 mutate(game_id = row_number(), Wk = as.numeric(Wk)) # %>% filter(Round == "Regular season")





# neue Funktion definieren, die die Siegwahrscheinlichkeiten berechnet
winprob <- function(team1, team2) {
 lt1 <- length(team1)
 lt2 <- length(team2)
 resultp <- c(0, 0, 0)
 names(resultp) <- c("Team 1", "X", "Team 2")
 if (lt1 == 0 & lt2 == 0) {
  resultp[[2]] <- 1
  return(resultp)
 }
 
 if (lt1 > lt2) {
  gt1 <- dpoisbinom(seq(from = 0, to = lt2), team1)
  gt1[[lt2 + 2]] <- 1 - sum(gt1)
  gt2 <- dpoisbinom(seq(from = 0, to = lt2), team2)
  
  resultp[[2]] <- sum(gt1[-(lt2 + 2)] * gt2)
  if (lt2 > 0) {
   resultp[[3]] <- sum(vapply(2:(lt2 + 1), function(x) gt2[[x]] * sum(gt1[seq_len(x - 1)]),
                              FUN.VALUE = 2))
  }
  resultp[[1]] <- 1 - sum(resultp)
  
 } else {
  gt1 <- dpoisbinom(seq(from = 0, to = lt1), team1)
  gt2 <- dpoisbinom(seq(from = 0, to = lt1), team2)
  gt2[[lt1 + 2]] <- 1 - sum(gt2)
  
  resultp[[2]] <- sum(gt1 * gt2[-(lt1 + 2)])
  if (lt1 > 0) {
   resultp[[1]] <- sum(vapply(2:(lt1 + 1), function(x) gt1[[x]] * sum(gt2[seq_len(x - 1)]),
                              FUN.VALUE = 2))
  }
  resultp[[3]] <- 1 - sum(resultp)
 }
 resultp
}

#### bestehende Tabellen updaten ####
# Einlesen von abgespeicherten Tabellendaten
league_table <- readRDS(filename_tab)

# neue Spiele herausfinden
newgames <- inner_join(league_matches %>% filter(!is.na(Home_xG)),
                       league_table %>% filter(is.na(xGF) & Location == "Home") %>% select(Wk, Squad),
                       by = c("Wk", "Home" = "Squad"))
shots_newgames <- fb_match_shooting(newgames$MatchURL) %>%
 mutate(xG = as.numeric(xG), Date = as.Date(Date))

shots_newgames <- newgames %>% pivot_longer(Home | Away, values_to = "Squad", names_to = "location") %>%
 select(Date, Squad, game_id) %>%
 right_join(shots_newgames, by = c("Date", "Squad"), multiple = "all")

#' Vorbereitung frs Updaten (groe hnlichkeit mit einigen
#' Code-Abschnitt beim ersten Aufsetzen)
ms_situation <- which(shots_newgames$Event_SCA_1 == "Shot" |
                       (shots_newgames$Event_SCA_2 == "Shot" &
                         !shots_newgames$Event_SCA_1 %in% c("Fouled", "Pass (Dead)")))
if (length(ms_situation) > 0) {
 xg_temp <- shots_newgames$xG
 for (i in ms_situation) {
  xg_temp[[i]] <- 1 - (1 - xg_temp[[i-1]]) * (1 - xg_temp[[i]])
 }
 shots_newgames <- shots_newgames %>% mutate(xG = xg_temp)
 shots_newgames <- shots_newgames %>% filter(!row_number() %in% (ms_situation - 1))
}

newgames <- newgames %>%
 mutate(vapply(game_id, function(id) {
  wp <- winprob((shots_newgames %>% filter(game_id == id & Home_Away == "Home"))$xG,
                (shots_newgames %>% filter(game_id == id & Home_Away == "Away"))$xG)
  c("Home_xPTS" = 3*wp[[1]] + wp[[2]], "Away_xPTS" = 3*wp[[3]] + wp[[2]])
 }, FUN.VALUE = numeric(2)) %>% t() %>% as_tibble())

newgames <- newgames %>% 
 mutate(Pts_Home = ifelse(HomeGoals > AwayGoals, 3,
                          ifelse(AwayGoals > HomeGoals, 0, 1)),
        Pts_Away = ifelse(Pts_Home == 3, 0, ifelse(Pts_Home == 1, 1, 3))) %>%
 pivot_longer(cols = c("Home", "Away"), values_to = "Squad", names_to = "Location") %>%
 mutate(GF = ifelse(Location == "Home", HomeGoals, AwayGoals),
        GA = ifelse(Location == "Home", AwayGoals, HomeGoals),
        xGF = ifelse(Location == "Home", Home_xG, Away_xG),
        xGA = ifelse(Location == "Home", Away_xG, Home_xG),
        Pts = ifelse(Location == "Home", Pts_Home, Pts_Away),
        xPts = ifelse(Location == "Home", Home_xPTS, Away_xPTS)) %>%
 group_by(Squad) %>%
 mutate(lastWk = min(Wk) - 1,
        GF = cumsum(GF) + league_table[league_table$Wk == lastWk & league_table$Squad %in% Squad, ]$GF,
        GA = cumsum(GA) + league_table[league_table$Wk == lastWk & league_table$Squad %in% Squad, ]$GA,
        GDiff = GF - GA,
        Pts = cumsum(Pts) + league_table[league_table$Wk == lastWk & league_table$Squad %in% Squad, ]$Pts,
        xGF = cumsum(xGF) + league_table[league_table$Wk == lastWk & league_table$Squad %in% Squad, ]$xGF,
        xGA = cumsum(xGA) + league_table[league_table$Wk == lastWk & league_table$Squad %in% Squad, ]$xGA,
        xGDiff = xGF - xGA,
        xPts = cumsum(xPts)+ league_table[league_table$Wk == lastWk & league_table$Squad %in% Squad, ]$xPts) %>%
 ungroup() %>% select(Wk, Squad, Location, GF, GA, Pts, xGF, xGA, xPts, GDiff, xGDiff)

# updaten der Tabelle
league_table <- rows_update(league_table, newgames, by = c("Wk", "Squad")) %>%
 arrange(Wk, desc(Pts), desc(GDiff), desc(GF), desc(xPts))

# Abspeichern der Tabellendaten
saveRDS(league_table, filename_tab)
