library(gt)
library(dplyr)

filename_tab <- "BL2324_xPTSTabelle.rds"
filename_vis <- "BL2324_xPTSTabelle_vis.html"




#### Visualisierung league_table ####
league_table <- readRDS(filename_tab)

#' wenn die Tabelle aus einer bestimmten Woche visualisiert werden soll
xtab_gt <- league_table %>%
 filter(Wk == 11)
#' wenn der letzte Tabellenstand visualisiert werden soll
xtab_gt <- league_table %>%
 arrange(desc(Wk), Pts, GDiff, GF, xPts) %>%
 filter(!is.na(Pts)) %>% filter(!duplicated(Squad)) %>%
 arrange(desc(Pts), desc(GDiff), desc(GF), desc(xPts)) 

# Bereich fr die Visualisierung der Farbskala bei den xPts
dom_range <- max(abs(league_table$Pts - league_table$xPts), na.rm = TRUE)

# Erstellen der Visualisierung der Ligatabelle
xtab_gt <- xtab_gt %>%
 mutate(Rk = row_number(), xPtsDiff = Pts - xPts) %>%
 rename(MP = Wk, Team = Squad) %>%
 select(Rk, Team, MP, GF, GA, GDiff, Pts, xGF, xGA, xGDiff, xPts, xPtsDiff) %>%
 gt() %>%
 data_color(columns = xPtsDiff, target_columns = xPts, palette = "RdBu",
            method = "numeric", domain = c(-dom_range, dom_range),
            reverse = TRUE) %>%
 cols_hide(xPtsDiff) %>%
 tab_spanner("Tabelle", columns = c("MP", "GF", "GA", "GDiff", "Pts")) %>%
 tab_spanner("xG Tabelle", columns = c("xGF", "xGA", "xGDiff", "xPts")) %>%
 fmt_number(xPts) %>% opt_stylize(style = 5) %>%
 tab_style(list(cell_fill("#04162a")),
           list(cells_column_spanners(), cells_column_labels())) %>%
 tab_source_note("data source: fbref.com") %>%
 cols_width(starts_with("G") ~ px(45), starts_with("xG") ~ px(45))

xtab_gt

# Abspeichern der Visualisierung
gtsave(xtab_gt, filename_vis)                        
